% Example usage of the CircularArc class
clear; close all; clc;

% North-East
pc = [10;5]; % [x,y]
R = 15;
alpha0 = pi/2;
alpha_del = -pi;
circular = CircularArc(pc, R, alpha0, alpha_del);

% Length
disp("L = " + circular.L);
f = @(t) tangentVectorNorm(circular, t);
integral(f, 0, circular.L)

% XY-plane
figure(); hold on;
s = 0:0.1:circular.L;
p = circular.p(s);
plot(p(2,:), p(1,:), 'b');
plot(p(2,1), p(1,1), 'gx');
plot(p(2,end), p(1,end), 'rx');
s = 0:circular.L/5:circular.L;
p = circular.p(s);
v = circular.v(s);
quiver(p(2,:), p(1,:), v(2,:), v(1,:), 0, 'k');
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
legend("Circular arc", "Start point", "End point", "Tangent vectors");

% Course angle
figure();
s = 0:0.1:circular.L;
chi = circular.chi(s);
plot(s, chi);
grid;
ylabel("\chi Course angle [rad]");
xlabel("s Path length [m]");

% Curvature
figure();
s = 0:0.1:circular.L;
kappa = circular.kappa(s);
plot(s, kappa);
grid;
ylabel("\kappa Curvature [m^{-1}]");
xlabel("s Path length [m]");


function norm = tangentVectorNorm(curve, s)
    v = curve.v(s);
    norm = sqrt(v(1,:).^2 + v(2,:).^2);
end
