% Example usage of the EulerSpiral class
clear; close all; clc;

% p0 = [10;5];
% chi0 = -pi/4;
% kappa0 = 0.5;
% kappa_del = -0.5;
% w = 3;
% spiral = EulerSpiral(p0, chi0, kappa0, kappa_del, w);

% Fundamental Euler spiral
p0 = [0;0];
chi0 = 0;
kappa0 = 0;
%kappa_del = 0.3;
kappa_del = 1/19.074963133938585; % curvature change of Euler spiral when phi_max = 60 deg, Va=18 m/s and p_des=120 deg/s
%w = 3;
w = 13.102468019630777; % scale factor of Euler spiral when [...]^
spiral = EulerSpiral(p0, chi0, kappa0, kappa_del, w);

% Length
disp("L = " + spiral.L);
f = @(t) tangentVectorNorm(spiral, t);
integral(f, 0, spiral.L)

% Rotation and reflection matrix
Rz = @(theta) [cos(theta), -sin(theta);
               sin(theta), cos(theta)];
Ref = @(rho) [1, 0;
              0, rho];

v_spiral_s_e = fresnel_integrals(w, spiral.L);

% % Compute start or end point without solving Fresnel integrals again
% if kappa0==0
%     p1 = p0 + Rz(chi0)*Ref(sign(kappa_del))*v_spiral_s_e;
% else
%     p1 = spiral.p(spiral.L);
%     p0 = p1 - Rz(spiral.chi(spiral.L))*Ref(sign(kappa_del))*v_spiral_s_e;
% end
    
% XY-plane
figure(); hold on;
s = 0:0.01:spiral.L;
p = spiral.p(s);
plot(p(2,:), p(1,:), 'k');
plot(p(2,1), p(1,1), 'gx');
plot(p(2,end), p(1,end), 'rx');
% plot(p0(2), p0(1), 'go');
% plot(p1(2), p1(1), 'ro');
% s_v = 0:spiral.L/5:spiral.L;
% p_v = spiral.p(s_v);
% v = spiral.v(s_v);
% quiver(p_v(2,:), p_v(1,:), v(2,:), v(1,:), 0, 'k');
if kappa0~=0
    R0 = 1/abs(kappa0);
    pc0 = spiral.p(0) + R0*[0, -sign(kappa0);
                           sign(kappa0),  0]*spiral.v(0);
    circle0 = CircularArc(pc0, R0, 0, 2*pi);
    
    s = 0:0.01:circle0.L;
    p = circle0.p(s);
    plot(p(2,:), p(1,:), 'm:');
end
if (kappa0+kappa_del)~=0
    kappa1 = kappa0+kappa_del;
    R1 = 1/abs(kappa1);
    pc1 = spiral.p(spiral.L) + R1*[0, -sign(kappa1);
                                   sign(kappa1),  0]*spiral.v(spiral.L);
    circle1 = CircularArc(pc1, R1, 0, 2*pi);
    
    s = 0:0.01:circle1.L;
    p = circle1.p(s);
    plot(p(2,:), p(1,:), 'm:');
end
grid;
box on;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
%legend("Euler spiral", "Start point", "End point", "Tangent vectors", "Turning circle");
legend("Euler spiral", "Start point", "End point", "Turning circle");

% Course angle
figure();
s = 0:0.1:spiral.L;
chi = spiral.chi(s);
plot(s, chi);
grid;
ylabel("\chi Course angle [rad]");
xlabel("s Path length [m]");

% Curvature
figure();
s = 0:0.1:spiral.L;
kappa = spiral.kappa(s);
plot(s, kappa);
grid;
ylabel("\kappa Curvature [m^{-1}]");
xlabel("s Path length [m]");


function norm = tangentVectorNorm(curve, s)
    v = curve.v(s);
    norm = sqrt(v(1,:).^2 + v(2,:).^2);
end
