% Example usage of the LineSegment class
clear; close all; clc;

% North-East
p0 = [0;0]; % [x, y]
p1 = [-10;-1];
line = LineSegment(p0, p1);

% Length
disp("L = " + line.L);
f = @(t) tangentVectorNorm(line, t);
integral(f, 0, line.L)

% XY-plane
figure(); hold on;
s = 0:0.1:line.L;
p = line.p(s);
plot(p(2,:), p(1,:), 'b');
plot(p(2,1), p(1,1), 'gx');
plot(p(2,end), p(1,end), 'rx');
s = 0:line.L/5:line.L;
p = line.p(s);
v = line.v(s);
quiver(p(2,:), p(1,:), v(2,:), v(1,:), 0, 'k');
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
legend("Line", "Start point", "End point", "Tangent vectors");

% Course angle
figure();
s = 0:0.1:line.L;
chi = line.chi(s);
plot(s, chi);
grid;
ylabel("\chi Course angle [rad]");
xlabel("s Path length [m]");

% Curvature
figure();
s = 0:0.1:line.L;
kappa = line.kappa(s);
plot(s, kappa);
grid;
ylabel("\kappa Curvature [m^{-1}]");
xlabel("s Path length [m]");


function norm = tangentVectorNorm(curve, s)
    v = curve.v(s);
    norm = sqrt(v(1,:).^2 + v(2,:).^2);
end
