% Example usage of the Spline2D class
clear; close all; clc;

%% LineSegment
p0 = [0;0]; % [x, y]
p1 = [-10;-1];
line = LineSegment(p0, p1);
spline_line = Spline2D(line);

% XY-plane
figure(); hold on;
s = 0:0.1:line.L;
p = line.p(s);
plot(p(2,:), p(1,:), 'k');
p_spline = spline_line.S(s);
plot(p_spline(2,:), p_spline(1,:), 'b');
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
legend("Line", "Spline");

% Curvature
figure(); hold on;
s = 0:0.1:line.L;
kappa = line.kappa(s);
plot(s, kappa, 'k');
S_der = spline_line.S_der(s);
S_dder = spline_line.S_dder(s);
kappa_spline = (S_der(1,:).*S_dder(2,:)-S_der(2,:).*S_dder(1,:))./((S_der(1,:).^2 + S_der(2,:).^2).^(3/2));
plot(s, kappa_spline, 'b');
grid;
ylabel("\kappa Curvature [m^{-1}]");
xlabel("s Path length [m]");
legend("Line", "Spline");

%% Circular Arc

pc = [10;5]; % [x,y]
R = 15;
alpha0 = pi;
alpha_del = pi;
circular = CircularArc(pc, R, alpha0, alpha_del);
spline_circular = Spline2D(circular);

s = 0:0.1:circular.L;
chi = circular.chi(s);
S_der = spline_circular.S_der(s);
chi_spline = atan2(S_der(2,:), S_der(1,:));

% XY-plane
figure(); hold on;
s = 0:0.1:circular.L;
p = circular.p(s);
plot(p(2,:), p(1,:), 'k');
p_spline = spline_circular.S(s);
plot(p_spline(2,:), p_spline(1,:), 'b');
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
legend("Circular arc", "Spline");

% Curvature
figure(); hold on;
s = 0:0.1:circular.L;
kappa = circular.kappa(s);
plot(s, kappa, 'k');
S_der = spline_circular.S_der(s);
S_dder = spline_circular.S_dder(s);
kappa_spline = (S_der(1,:).*S_dder(2,:)-S_der(2,:).*S_dder(1,:))./((S_der(1,:).^2 + S_der(2,:).^2).^(3/2));
plot(s, kappa_spline);
grid;
box on;
ylabel("Curvature \kappa [m^{-1}]");
xlabel("Path length s [m]");
legend("Circular arc", "Spline");

%% Euler Spiral

p0 = [10;5];
chi0 = -pi/2;
%kappa0 = 1/19.074963133938585;
kappa0 = 0;
%kappa_del = 0.3;
kappa_del = 1/19.074963133938585; % curvature change of Euler spiral when phi_max = 60 deg, Va=18 m/s and p_des=120 deg/s
%w = 3;
w = 13.102468019630777; % scale factor of Euler spiral when [...]^
spiral = EulerSpiral(p0, chi0, kappa0, kappa_del, w);
spline_spiral = Spline2D(spiral);

s = 0:0.1:spiral.L;
chi = spiral.chi(s);
S_der = spline_spiral.S_der(s);
chi_spline = atan2(S_der(2,:), S_der(1,:));

% XY-plane
figure(); hold on;
s = 0:0.01:spiral.L;
p = spiral.p(s);
plot(p(2,:), p(1,:), 'k');
p_spline = spline_spiral.S(s);
plot(p_spline(2,:), p_spline(1,:), 'b');
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
legend("Euler spiral", "Spline");

% Curvature
figure(); hold on;
s = 0:0.1:spiral.L;
kappa = spiral.kappa(s);
plot(s, kappa, 'k');
S_der = spline_spiral.S_der(s);
S_dder = spline_spiral.S_dder(s);
kappa_spline = (S_der(1,:).*S_dder(2,:)-S_der(2,:).*S_dder(1,:))./((S_der(1,:).^2 + S_der(2,:).^2).^(3/2));
plot(s, kappa_spline);
grid;
box on;
ylabel("Curvature \kappa [m^{-1}]");
xlabel("Path length s [m]");
legend("Euler spiral", "Spline");
