% Debug script for the Dubins2D class
clear; close all; clc;

% UAV parameters
p0 = [-10; -1];         % Initial 2D position
chi0 = deg2rad(-45);    % Initial course angle [rad]
chi_n = deg2rad(90);    % Final course angle (at last waypoint) [rad]
Va = 18;                % Cruise airspeed [m/s]
phi_max = deg2rad(60);  % Max roll angle [rad]

g = 9.80665;                        % Gravitational acceleration [m/s^2]
chi_dot_max = g/Va*tan(phi_max);    % Turn rate with max roll angle in a coordinated turn [deg/s]
R = Va/chi_dot_max;                 % Radius of circular arcs [m]  

% Constant altitude waypoints
load('WP.mat');
p_wp = [p0, WP];
n = length(p_wp);

% Dubins2D
path = Dubins2D(chi0, chi_n, R, p_wp);

% XY-plane
figure(); hold on;
% Waypoints
for i=1:n
    plot(path.p_wp(2,i), path.p_wp(1,i), 'ks');
end
% Path
s = 0:0.01:path.L;
p = path.p(s);
plot(p(2,:), p(1,:), 'k.', 'MarkerSize', 2);
% DEBUG
for i=1:n
    % Lineear entering tangent vector, exiting tangent vector, waypoint vector
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_in(2,i), Va*path.debug.v_in(1,i), 0, 'r--');
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_out(2,i), Va*path.debug.v_out(1,i), 0, 'g--');
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_wp(2,i), Va*path.debug.v_wp(1,i), 0, 'k--');
    
    % Circle to circular arc
    plot(path.debug.pc(2,i), path.debug.pc(1,i), 'm*');
    circle = CircularArc(path.debug.pc(:,i), path.R, 0, 2*pi);
    s = 0:0.1:circle.L;
    p_circ = circle.p(s);
    plot(p_circ(2,:), p_circ(1,:), 'm:');
    
    % Wheel-over point, pull-out point
    plot(path.debug.p_wo(2,i), path.debug.p_wo(1,i), 'rx');
    plot(path.debug.p_po(2,i), path.debug.p_po(1,i), 'gx');
%     quiver(path.debug.p_wo(2,i), path.debug.p_wo(1,i), Va*path.debug.v_wo(2,i), Va*path.debug.v_wo(1,i), 0, 'r--');
%     quiver(path.debug.p_po(2,i), path.debug.p_po(1,i), Va*path.debug.v_po(2,i), Va*path.debug.v_po(1,i), 0, 'g--');
end
grid;
box on;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
% Hacky legend
h = zeros(8, 1);
h(1) = plot(NaN, NaN, 'k');
h(2) = plot(NaN, NaN, 'ks');
h(3) = plot(NaN, NaN, 'r--');
h(4) = plot(NaN, NaN, 'g--');
h(5) = plot(NaN, NaN, 'k--');
h(6) = plot(NaN, NaN, 'm*');
h(7) = plot(NaN, NaN, 'rx');
h(8) = plot(NaN, NaN, 'gx');
legend(h, "$\mathcal{P}$", "$\mathbf{p}_{wp}$", "$\mathbf{v}_{in}$", "$\mathbf{v}_{out}$", "$\mathbf{v}_{wp}$", "$\mathbf{p}_{c}$", "$\mathbf{p}_{wo}$", "$\mathbf{p}_{po}$", "Interpreter", "latex");
