% Debug script for the ExtendedDubins2D class
clear; close all; clc;

% UAV parameters
p0 = [-10; -1];         % Initial 2D position
chi0 = deg2rad(-45);    % Initial course angle [rad]
chi_n = deg2rad(90);    % Final course angle (at last waypoint) [rad]
Va = 18;                % Cruise airspeed [m/s]
phi_max = deg2rad(60);  % Max roll angle [rad]
p_des = deg2rad(120);   % Desired roll rate [rad/s]

g = 9.80665;                        % Gravitational acceleration [m/s^2]
chi_dot_max = g/Va*tan(phi_max);    % Turn rate with max roll angle in a coordinated turn [deg/s]
R = Va/chi_dot_max;                 % Radius of circular arcs [m]  

L_spiral = Va*(phi_max/p_des);      % Length of Euler spirals

% Constant altitude waypoints
load('WP.mat');
p_wp = [p0, WP];
n = length(p_wp);

% ExtendedDubins2D
path = ExtendedDubins2D(chi0, chi_n, R, L_spiral, p_wp);

% XY-plane
figure(); hold on;
% Waypoints
for i=1:n
    plot(path.p_wp(2,i), path.p_wp(1,i), 'ks');
end
% Path
% NB: Time consuming as Fresnel integrals are solved for each s, the path
% must be spline approximated before being useful
% s = 0:0.01:path.L;
% p = path.p(s);
% plot(p(2,:), p(1,:), 'k.', 'MarkerSize', 2);
% DEBUG
for i=1:n
    % Entering tangent vector, exiting tangent vector, waypoint vector
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_in(2,i), Va*path.debug.v_in(1,i), 0, 'r--');
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_out(2,i), Va*path.debug.v_out(1,i), 0, 'g--');
    quiver(path.p_wp(2,i), path.p_wp(1,i), Va*path.debug.v_wp(2,i), Va*path.debug.v_wp(1,i), 0, 'k--');
    
    % Circle to circular arc
    plot(path.debug.pc(2,i), path.debug.pc(1,i), 'm*');
    circle = CircularArc(path.debug.pc(:,i), path.R, 0, 2*pi);
    s = 0:0.1:circle.L;
    p = circle.p(s);
    plot(p(2,:), p(1,:), 'm:');
    
    % Circle to spiral
    circle_spiral = CircularArc(path.debug.pc(:,i), path.debug.R_spiral, 0, 2*pi);
    s = 0:0.1:circle_spiral.L;
    p = circle_spiral.p(s);
    plot(p(2,:), p(1,:), 'c:');
    
    % Makeshift wheel-over point, pull-out point
    plot(path.debug.p_wo(2,i), path.debug.p_wo(1,i), 'rx');
    plot(path.debug.p_po(2,i), path.debug.p_po(1,i), 'gx');
    
    % Wheel-over-start and wheel-over-end points, pull-out-start and pull-out-end points
    plot(path.debug.p_wo_s(2,i), path.debug.p_wo_s(1,i), 'ro');
    plot(path.debug.p_wo_e(2,i), path.debug.p_wo_e(1,i), 'rd');
    
    plot(path.debug.p_po_s(2,i), path.debug.p_po_s(1,i), 'gd');
    plot(path.debug.p_po_e(2,i), path.debug.p_po_e(1,i), 'go');
end
grid;
box on;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
% Hacky legend
h = zeros(8, 1);
h(1) = plot(NaN, NaN, 'k');
h(2) = plot(NaN, NaN, 'ks');
h(3) = plot(NaN, NaN, 'r--');
h(4) = plot(NaN, NaN, 'g--');
h(5) = plot(NaN, NaN, 'k--');
h(6) = plot(NaN, NaN, 'm*');
h(7) = plot(NaN, NaN, 'rx');
h(8) = plot(NaN, NaN, 'gx');
h(9) = plot(NaN, NaN, 'ro');
h(10) = plot(NaN, NaN, 'rd');
h(11) = plot(NaN, NaN, 'gd');
h(12) = plot(NaN, NaN, 'go');
legend(h, "$\mathcal{P}$", "$\mathbf{p}_{wp}$", "$\mathbf{v}_{in}$", "$\mathbf{v}_{out}$", "$\mathbf{v}_{wp}$", "$\mathbf{p}_{c}$", "$\mathbf{p}_{wo}$", "$\mathbf{p}_{po}$",  "$\mathbf{p}_{wo-s}$", "$\mathbf{p}_{wo-e}$", "$\mathbf{p}_{po-s}$", "$\mathbf{p}_{po-e}$", "Interpreter", "latex");
