% Example usage of the Dubins2D class
clear; close all; clc;

% UAV parameters
p0 = [-10; -1];         % Initial 2D position
chi0 = deg2rad(-45);    % Initial course angle [rad]
chi_n = deg2rad(90);    % Final course angle (at last waypoint) [rad]
Va = 18;                % Cruise airspeed [m/s]
phi_max = deg2rad(60);  % Max roll angle [rad]

g = 9.80665;                        % Gravitational acceleration [m/s^2]
chi_dot_max = g/Va*tan(phi_max);    % Turn rate with max roll angle in a coordinated turn [deg/s]
R = Va/chi_dot_max;                 % Radius of circular arcs [m]  

% Constant altitude waypoints
load('WP.mat');

p_wp = [p0, WP];
n = length(p_wp);

% Dubins2D
path = Dubins2D(chi0, chi_n, R, p_wp);

% What part of the path to plot below
s_start = path.table(1).s0;
s_end = path.table(end).s1;
s = s_start:0.01:s_end;
p = path.p(s);

% XY-plane
figure(); hold on;
% Waypoints
for i=1:n
    plot(path.p_wp(2,i), path.p_wp(1,i), 'ks');
end
% Path
plot(p(2,:), p(1,:), 'b.', 'MarkerSize', 2);
% % Tangent vectors
% s_v = s_start:5:s_end;
% p_v = path.p(s_v);
% v = path.v(s_v);
% for i=1:length(v)
%     quiver(p_v(2,i), p_v(1,i), Va*v(2,i), Va*v(1,i), 0, 'k--');
% end
% Initial and final course angle
quiver(path.p_wp(2,1), path.p_wp(1,1), Va*sin(chi0), Va*cos(chi0), 0, 'g--');
quiver(path.p_wp(2,n), path.p_wp(1,n), Va*sin(chi_n), Va*cos(chi_n), 0, 'r--');
grid;
box on;
axis equal;
xlabel("East y [m]");
ylabel("North x [m]");
% Hacky legend
h = zeros(3, 1);
h(1) = plot(NaN, NaN, 'b');
h(2) = plot(NaN, NaN, 'ks');
h(3) = plot(NaN, NaN, 'g--');
h(4) = plot(NaN, NaN, 'r--');
legend(h, "$\mathcal{P}$", "$\mathbf{p}_{wp}$", "$\chi_0$", "$\chi_n$", "Interpreter", "latex");
ylim([-20, 410]);

figure();
% Course angle
subplot(2,1,1);
chi = path.chi(s);
plot(s, chi);
for i=1:length(path.table)-1
    if ~strcmp(class(path.table(i).subpath), class(path.table(i+1).subpath))
        xline(path.table(i).s1, 'k:');
    end
end
%grid;
ylabel("Course angle \chi [rad]");
% Curvature
subplot(2,1,2); hold on;
kappa = path.kappa(s);
plot(s, kappa, '.', 'MarkerSize', 3);
for i=1:length(path.table)-1
    if ~strcmp(class(path.table(i).subpath), class(path.table(i+1).subpath))
        xline(path.table(i).s1, 'k:');
    end
end
%grid;
box on;
ylabel("Curvature \kappa [m^{-1}]");
xlabel("Path length s [m]");
ylim([-0.06, 0.06]);
