% Debug script for the ExtendedDubins3D class
clear; close all; clc;

% NB: A bit timeconsuming as Fresnel integrals are solved for each s, the path
% must be spline approximated before being useful.

% UAV parameters
p0 = [-10; -1; -100];   % Initial 3D position
chi0 = deg2rad(-45);    % Initial course angle [rad]
chi_n = deg2rad(90);    % Final course angle (at last waypoint) [rad]
gamma0 = deg2rad(0);    % Initial flight path angle [rad]
gamma_n = deg2rad(0);   % Final fligth path angle [rad]
Va = 18;                % Cruise airspeed [m/s]
phi_max = deg2rad(60);  % Max roll angle [rad]
theta_max = deg2rad(30);    % Max pitch angle [rad]
p_des = deg2rad(120);   % Desired roll rate [rad/s]
q_des = deg2rad(60);   % Desired pitch rate [rad/s]

g = 9.80665;                        % Gravitational acceleration [m/s^2]
chi_dot_max = g/Va*tan(phi_max);    % Turn rate with max roll angle in a coordinated turn [deg/s]
R_xy = Va/chi_dot_max;              % Radius of horizontal circular arcs [m]  

L_spiral = Va*(phi_max/p_des);      % Length of Euler spirals

R_hs = Va/q_des;                    % Radius of vertical circular arcs [m]

% Constant altitude waypoints
load('WP3D.mat');
p_wp = [p0, WP];
n = length(p_wp);

% ExtendedDubins3D
path = ExtendedDubins3D(chi0, chi_n, R_xy, L_spiral, R_hs, theta_max, p_wp);

% X-Y plane
figure(); hold on;
% Waypoints
for i=1:n
    plot(path.path_xy.p_wp(2,i), path.path_xy.p_wp(1,i), 'ks');
end
% Subpaths
for i=1:length(path.path_xy.table)
    s = 0:0.01:path.path_xy.table(i).subpath.L;
    p = path.path_xy.table(i).subpath.p(s);
    plot(p(2,:), p(1,:), 'k.', 'MarkerSize', 2);
end
% DEBUG
for i=1:n
    % Entering tangent vector, exiting tangent vector, waypoint vector
    quiver(path.path_xy.p_wp(2,i), path.path_xy.p_wp(1,i), Va*path.path_xy.debug.v_in(2,i), Va*path.path_xy.debug.v_in(1,i), 0, 'r--');
    quiver(path.path_xy.p_wp(2,i), path.path_xy.p_wp(1,i), Va*path.path_xy.debug.v_out(2,i), Va*path.path_xy.debug.v_out(1,i), 0, 'g--');
    quiver(path.path_xy.p_wp(2,i), path.path_xy.p_wp(1,i), Va*path.path_xy.debug.v_wp(2,i), Va*path.path_xy.debug.v_wp(1,i), 0, 'k--');
    
    % Circle to circular arc
    plot(path.path_xy.debug.pc(2,i), path.path_xy.debug.pc(1,i), 'm*');
    circle = CircularArc(path.path_xy.debug.pc(:,i), path.path_xy.R, 0, 2*pi);
    s = 0:0.1:circle.L;
    p = circle.p(s);
    plot(p(2,:), p(1,:), 'm:');
    
    % Circle to spiral
    circle_spiral = CircularArc(path.path_xy.debug.pc(:,i), path.path_xy.debug.R_spiral, 0, 2*pi);
    s = 0:0.1:circle_spiral.L;
    p = circle_spiral.p(s);
    plot(p(2,:), p(1,:), 'c:');
    
    % Makeshift wheel-over point, pull-out point
    plot(path.path_xy.debug.p_wo(2,i), path.path_xy.debug.p_wo(1,i), 'rx');
    plot(path.path_xy.debug.p_po(2,i), path.path_xy.debug.p_po(1,i), 'gx');
    
    % Wheel-over-start and wheel-over-end points, pull-out-start and pull-out-end points
    plot(path.path_xy.debug.p_wo_s(2,i), path.path_xy.debug.p_wo_s(1,i), 'ro');
    plot(path.path_xy.debug.p_wo_e(2,i), path.path_xy.debug.p_wo_e(1,i), 'rd');
    
    plot(path.path_xy.debug.p_po_s(2,i), path.path_xy.debug.p_po_s(1,i), 'gd');
    plot(path.path_xy.debug.p_po_e(2,i), path.path_xy.debug.p_po_e(1,i), 'go');
end
grid;
axis equal;
xlabel("y East [m]");
ylabel("x North [m]");
% Hacky legend
h = zeros(8, 1);
h(1) = plot(NaN, NaN, 'k');
h(2) = plot(NaN, NaN, 'ks');
h(3) = plot(NaN, NaN, 'r--');
h(4) = plot(NaN, NaN, 'g--');
h(5) = plot(NaN, NaN, 'k--');
h(6) = plot(NaN, NaN, 'm*');
h(7) = plot(NaN, NaN, 'rx');
h(8) = plot(NaN, NaN, 'gx');
h(9) = plot(NaN, NaN, 'ro');
h(10) = plot(NaN, NaN, 'rd');
h(11) = plot(NaN, NaN, 'gd');
h(12) = plot(NaN, NaN, 'go');
legend(h, "$\mathcal{P}$", "$\mathbf{p}_{wp}$", "$\mathbf{v}_{in}$", "$\mathbf{v}_{out}$", "$\mathbf{v}_{wp}$", "$\mathbf{p}_{c}$", "$\mathbf{p}_{wo}$", "$\mathbf{p}_{po}$",  "$\mathbf{p}_{wo-s}$", "$\mathbf{p}_{wo-e}$", "$\mathbf{p}_{po-s}$", "$\mathbf{p}_{po-e}$", "Interpreter", "latex");

% H-S_h plane
figure(); hold on;
% Waypoints
for i=1:n
    plot(path.path_hs.p_wp(2,i), path.path_hs.p_wp(1,i), 'ks');
end
% Subpaths
s = 0:0.01:path.path_hs.L;
p_hs = path.path_hs.p(s);
plot(p_hs(2,:), p_hs(1,:), 'k.', 'MarkerSize', 2);
% DEBUG
for i=1:n
    % Entering tangent vector, exiting tangent vector, waypoint vector
    quiver(path.path_hs.p_wp(2,i), path.path_hs.p_wp(1,i), Va*path.path_hs.debug.v_in(2,i), Va*path.path_hs.debug.v_in(1,i), 0, 'r--');
    quiver(path.path_hs.p_wp(2,i), path.path_hs.p_wp(1,i), Va*path.path_hs.debug.v_out(2,i), Va*path.path_hs.debug.v_out(1,i), 0, 'g--');
    quiver(path.path_hs.p_wp(2,i), path.path_hs.p_wp(1,i), Va*path.path_hs.debug.v_wp(2,i), Va*path.path_hs.debug.v_wp(1,i), 0, 'k--');
    
    % Circle to circular arc
    plot(path.path_hs.debug.pc(2,i), path.path_hs.debug.pc(1,i), 'm*');
    circle = CircularArc(path.path_hs.debug.pc(:,i), path.path_hs.R, 0, 2*pi);
    s = 0:0.1:circle.L;
    p = circle.p(s);
    plot(p(2,:), p(1,:), 'm:');
    
    % Wheel-over point, pull-out point
    plot(path.path_hs.debug.p_wo(2,i), path.path_hs.debug.p_wo(1,i), 'rx');
    plot(path.path_hs.debug.p_po(2,i), path.path_hs.debug.p_po(1,i), 'gx');
end
grid;
axis equal;
xlabel("s_h Horizontal path length [m]");
ylabel("h Altitude [m]");
% Hacky legend
h = zeros(8, 1);
h(1) = plot(NaN, NaN, 'k');
h(2) = plot(NaN, NaN, 'ks');
h(3) = plot(NaN, NaN, 'r--');
h(4) = plot(NaN, NaN, 'g--');
h(5) = plot(NaN, NaN, 'k--');
h(6) = plot(NaN, NaN, 'm*');
h(7) = plot(NaN, NaN, 'rx');
h(8) = plot(NaN, NaN, 'gx');
legend(h, "$\mathcal{P}$", "$\mathbf{p}_{wp}$", "$\mathbf{v}_{in}$", "$\mathbf{v}_{out}$", "$\mathbf{v}_{wp}$", "$\mathbf{p}_{c}$", "$\mathbf{p}_{wo}$", "$\mathbf{p}_{po}$", "Interpreter", "latex");
